@@ -17,6 +17,7 @@
 QT_BEGIN_NAMESPACE
 QT_WARNING_PUSH
 QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
+QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")
 
 #if !defined(Q_WAYLAND_SERVER_SERVER_BUFFER_EXTENSION_EXPORT)
 #  if defined(QT_SHARED)
@@ -40,13 +41,13 @@ namespace QtWaylandServer {
         class Resource
         {
         public:
-            Resource() : server_buffer_object(0), handle(0) {}
+            Resource() : server_buffer_object(nullptr), handle(nullptr) {}
             virtual ~Resource() {}
 
             qt_server_buffer *server_buffer_object;
             struct ::wl_resource *handle;
 
-            struct ::wl_client *client() const { return handle->client; }
+            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
             int version() const { return wl_resource_get_version(handle); }
 
             static Resource *fromResource(struct ::wl_resource *resource);
@@ -66,8 +67,8 @@ namespace QtWaylandServer {
         QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
         const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }
 
-        bool isGlobal() const { return m_global != 0; }
-        bool isResource() const { return m_resource != 0; }
+        bool isGlobal() const { return m_global != nullptr; }
+        bool isResource() const { return m_resource != nullptr; }
 
         static const struct ::wl_interface *interface();
         static QByteArray interfaceName() { return interface()->name; }
@@ -85,6 +86,7 @@ namespace QtWaylandServer {
     private:
         static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
         static void destroy_func(struct ::wl_resource *client_resource);
+        static void display_destroy_func(struct ::wl_listener *listener, void *data);
 
         Resource *bind(struct ::wl_client *client, uint32_t id, int version);
         Resource *bind(struct ::wl_resource *handle);
@@ -99,6 +101,10 @@ namespace QtWaylandServer {
         Resource *m_resource;
         struct ::wl_global *m_global;
         uint32_t m_globalVersion;
+        struct DisplayDestroyedListener : ::wl_listener {
+            qt_server_buffer *parent;
+        };
+        DisplayDestroyedListener m_displayDestroyedListener;
     };
 }
 
